/*
Copyright 2007 Daniel Gonzalez Peña, Florentino Fernandez Riverola


This file is part of the AIBench Project. 

AIBench Project is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

AIBench Project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser Public License for more details.

You should have received a copy of the GNU Lesser Public License
along with AIBench Project.  If not, see <http://www.gnu.org/licenses/>.
*/

/*  
 * PluginActionProvider.java
 *
 * Created inside the SING research group (http://sing.ei.uvigo.es)
 * University of Vigo
 *
 * Created on 11/04/2009
 */
package es.uvigo.ei.sing.aibench.pluginmanager.gui;

import java.awt.Point;
import java.io.File;
import java.util.Hashtable;

import javax.swing.ButtonModel;

import org.jdesktop.swingx.renderer.CellContext;
import org.jdesktop.swingx.renderer.ComponentProvider;
import org.jdesktop.swingx.rollover.RolloverProducer;
import org.jdesktop.swingx.rollover.RolloverRenderer;
import org.platonos.pluginengine.Plugin;

import es.uvigo.ei.aibench.repository.info.PluginInfo;
import es.uvigo.ei.sing.aibench.pluginmanager.PluginManager;

/**
 * @author Miguel Reboiro Jato
 *
 */
public class PluginActionProvider extends ComponentProvider<PluginActionComponent> implements RolloverRenderer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private final static Hashtable<String, PluginActionComponent> UID_PAC = new Hashtable<String, PluginActionComponent>();
	private final static PluginActionComponent emptyPAC = new PluginActionComponent();


	public final static void pluginDownloaderChanged() {
		PluginActionProvider.UID_PAC.clear();
	}
	
	private final static PluginActionComponent getPluginActionComponent(String uid) {
		if (uid == null || !PluginManager.getInstance().isDownloaderActive())
			return PluginActionProvider.emptyPAC;
		
		if (!PluginActionProvider.UID_PAC.containsKey(uid)) {
			synchronized(PluginActionProvider.UID_PAC) {
				if (!PluginActionProvider.UID_PAC.containsKey(uid)) {
					PluginInfo info = PluginManager.getInstance().getDownloadPluginInfo(uid);
					if (info == null) {
						PluginActionProvider.UID_PAC.put(uid, PluginActionProvider.emptyPAC);
					} else {
						Plugin plugin = PluginManager.getInstance().getInstalledPlugin(uid);
						boolean hasUpdate = PluginManager.getInstance().hasUpdateAvailable(uid);
						if (plugin == null) { // Install
							PluginActionComponent pac = new PluginActionComponent(info);
							PluginManager.getInstance().getPluginDownloader().addDownloadListener(pac);
							PluginActionProvider.UID_PAC.put(uid, pac);
						} else if (hasUpdate) { // Update
							File file = new File(plugin.getPluginURL().getPath());
							PluginActionComponent pac = new PluginActionComponent(info, file.getName());
							PluginManager.getInstance().getPluginDownloader().addDownloadListener(pac);
							PluginActionProvider.UID_PAC.put(uid, pac);							
						} else {
							PluginActionProvider.UID_PAC.put(uid, PluginActionProvider.emptyPAC);
						}
					}
				}
			}
		}
		
		return PluginActionProvider.UID_PAC.get(uid);
	}

	/* (non-Javadoc)
	 * @see org.jdesktop.swingx.renderer.ComponentProvider#getRendererComponent(org.jdesktop.swingx.renderer.CellContext)
	 */
	@Override
	public PluginActionComponent getRendererComponent(CellContext context) {
		if (context.getValue() instanceof String) {
			String uid = (String) context.getValue();
			this.rendererComponent = PluginActionProvider.getPluginActionComponent(uid);
			this.rendererComponent.addContainer(context.getComponent());
		} else {
			this.rendererComponent = PluginActionProvider.emptyPAC;
		}
		return super.getRendererComponent(context);
	}
	
	
	/* (non-Javadoc)
	 * @see org.jdesktop.swingx.renderer.ComponentProvider#createRendererComponent()
	 */
	@Override
	protected PluginActionComponent createRendererComponent() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.jdesktop.swingx.renderer.ComponentProvider#configureState(org.jdesktop.swingx.renderer.CellContext)
	 */
	@Override
	protected void configureState(CellContext context) {
		ButtonModel model = this.rendererComponent.getButtonModel();

		if (model != null && context.getComponent() != null) {
			Point p = (Point) context.getComponent().getClientProperty(
					RolloverProducer.ROLLOVER_KEY);
			if (/* hasFocus || */(p != null && (p.x >= 0)
					&& (p.x == context.getColumn()) && (p.y == context.getRow()))) {
				if (!model.isRollover())
					model.setRollover(true);
			} else {
				if (model.isRollover())
					model.setRollover(false);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesktop.swingx.renderer.ComponentProvider#format(org.jdesktop.swingx.renderer.CellContext)
	 */
	@Override
	protected void format(CellContext context) {}


	/* (non-Javadoc)
	 * @see org.jdesktop.swingx.rollover.RolloverRenderer#doClick()
	 */
	public void doClick() {
		if (this.rendererComponent != null) {
			this.rendererComponent.doClick();
		}
	}


	/* (non-Javadoc)
	 * @see org.jdesktop.swingx.rollover.RolloverRenderer#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
//		if (this.rendererComponent != null) {
//			return this.rendererComponent.isEnabled();
//		} else {
//			return false;
//		}
	}

}
